package mailinglist;

import java.io.IOException;
import java.util.concurrent.locks.ReentrantLock;

import com.sun.grizzly.Controller;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultProtocolChainInstanceHandler;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.TCPSelectorHandler;

/**
 * @author kloh
 * 
 */
public class Server {

	private static Controller sController;

	private TCPSelectorHandler mSelectorHandler = null;

	private final DefaultProtocolChain mPROTOCOL_CHAIN;

	private Thread sServerThread;
	private final ReentrantLock sServerThreadLock = new ReentrantLock();

	public Server() {

		this.mPROTOCOL_CHAIN = new DefaultProtocolChain();

		this.mPROTOCOL_CHAIN.addFilter(new MyProtocolFilter());
		this.mPROTOCOL_CHAIN.addFilter(new ForwardFilter());

	}

	private void configureServiceDispatcherHandler(
			final int pServiceFileDownloadPort) {

		if (this.mSelectorHandler == null) {

			this.mSelectorHandler = new TCPSelectorHandler();

			this.mSelectorHandler.setPort(pServiceFileDownloadPort);

			this.mSelectorHandler
					.setProtocolChainInstanceHandler(new DefaultProtocolChainInstanceHandler() {
						@Override
						public ProtocolChain poll() {
							return Server.this.mPROTOCOL_CHAIN;
						}
					});

		}

	}

	private void configureController(final int pPort) {

		if (sController == null) {

			sController = new Controller();

			configureServiceDispatcherHandler(pPort);
			sController.addSelectorHandler(this.mSelectorHandler);

		}

	}

	public void start_GrizzlyConnector() {

		final int lServiceFileDownloadPort = 3407;

		configureController(lServiceFileDownloadPort);

		if (this.sServerThread == null) {
			this.sServerThreadLock.lock();
			try {
				if (this.sServerThread == null) {
					this.sServerThread = new Thread(new Runnable() {

						public void run() {
							System.out.println("Listening @ port "
									+ lServiceFileDownloadPort + ".");

							try {
								sController.start();
							} catch (final IOException e) {
								throw new RuntimeException(e);
							}

							System.out.println("Stopped listening @ port "
									+ lServiceFileDownloadPort + ".");
						}

					});
					this.sServerThread.setName(Server.class.getName());
					this.sServerThread.start();
				}
			} finally {
				this.sServerThreadLock.unlock();
			}
		}

	}

	public void stop_GrizzlyConnector() {

		if (sController != null) {
			try {
				sController.stop();
			} catch (final IOException e) {
				throw new RuntimeException(e);
			} finally {
				this.sServerThread.interrupt();
				this.sServerThread = null;
				this.mSelectorHandler = null;
				sController = null;
			}
		}

	}

}
